module.exports = function(grunt) {

    // Load all
    require('load-grunt-tasks')(grunt);

    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),

        // Build
        clean: {
            www: {
                src: ['www']
            },
            browserify: {
                src: ['b']
            }
        },
        copy: {
            prod: {
                files: [
                    {
                        expand: true,
                        cwd: 'src/',
                        src: [
                            'assets/**',
                            'index.html'
                        ],
                        dest: 'www',
                        filter: 'isFile'
                    },
                    {
                        expand: true,
                        src: [
                            'package.json',
                            'LICENSE'
                        ],
                        dest: 'www',
                        filter: 'isFile'
                    },
                    {
                        expand: true,
                        cwd: 'lib/',
                        src: [
                            'rbc.js'
                        ],
                        dest: 'www/lib',
                        filter: 'isFile'
                    }
                ]
            },
            dev: {
                files: [
                    {
                        expand: true,
                        cwd: 'src/',
                        src: [
                            '**'
                        ],
                        dest: 'www',
                        filter: 'isFile'
                    },
                    {
                        expand: true,
                        src: [
                            'package.json',
                            'LICENSE'
                        ],
                        dest: 'www',
                        filter: 'isFile'
                    },
                    {
                        expand: true,
                        cwd: 'lib/',
                        src: [
                            'rbc.js'
                        ],
                        dest: 'www/lib',
                        filter: 'isFile'
                    }
                ]
            }
        },
        cssmin: {
            default: {
                files: {
                    'www/app.css': ['src/app.css']
                }
            }
        },
        jshint: {
            ignore_warning: {
                options: {
                    '-W097': true,  // ignore: 'use strict' has to be wrapped in fn
                                        // browserify will wrap it
                    globals: {
                        moment: false,
                        window: false,
                        history: false,
                        document: false,
                        navigator: false,
                        console: false,
                        $: false,
                        Handlebars: false,
                        _: false,
                        Q: false,
                        setTimeout: false
                    },
                    ignores: ['src/**/*.css', 'src/**/*.html', 'src/**/*.png', 'src/**/*.jpg', 'src/**/icomoon/**', '**/*.md']
                },
                src: ['src/**']
            }
        },
        uglify: {
            options: {
                mangle: false
            },
            my_target: {
                files: {
                    'www/app.js': 'src/app.js'
                }
            }
        },
        jasmine: {
            src: [
                'browserified.js'
            ],
            options: {
                junit: {
                    path: 'tests/output/junit'
                },
                template: require('grunt-template-jasmine-istanbul'),
                templateOptions: {
                    coverage: 'tests/output/coverage/coverage.json',
                    report: [
                        {type: 'cobertura', options: {dir: 'tests/output/coverage/cobertura'}},
                        {type: 'lcov', options: {dir: 'tests/output/coverage/lcov'}}
                    ],
                    thresholds: {
                        'lines': 0,
                        'statements': 0,
                        'branches': 0,
                        'functions': 0
                    }
                }
            }
        },

        // Connect
        connect: {
            server: {
                options: {
                    port: 3000,
                    base: 'www',
                    keepalive: true
                }
            }
        }
    });

    grunt.registerTask('build',
        [
            'clean:www',
            'jshint',
            'copy:dev'
        ]
    );

    grunt.registerTask('prod-build',
        [
            'clean:www',
            'jshint',
            'copy:prod',
            'cssmin',
            'uglify'
        ]
    );

    grunt.registerTask('localhost', ['build', 'connect']);  // host locally
    grunt.registerTask('test', []);

};
