'use strict';

var data = [],
    type = '<%= type %>',
    details = <%= details %>;   // The app will show these attributes of the type

var uri;

// Get data and generate views

/**
 * Find the namespace that the logged in user belongs to.
 * @param uri
 * @returns {promise|callInfo.promise|Promise.promise}
 */
var getNamespace = function(uri) {

    var q = Q.defer();

    $.ajax({
        type: 'GET',
        url: uri + '/me',
        xhrFields: {
            withCredentials: true   // Needed for cookie authentication!
        },
        dataType: 'JSON'
    })
        .done(function(data, textStatus, jqXHR) {

            var namespace = data.authorizations[0].namespace;

            q.resolve(namespace);
        })
        .fail(function(jqXHR, textStatus, errorThrown) {

            q.reject();
        });

    return q.promise;
};


/**
 * Retrieve the first 20 records of the specified type.
 * @param uri
 * @param namespace
 * @param type
 * @returns {promise|callInfo.promise|Promise.promise}
 */
var getData = function(uri, namespace, type) {

    var q = Q.defer();

    $.ajax({
        type: 'GET',
        url: createUrl(uri, namespace, type),
        xhrFields: {
            withCredentials: true   // Needed for cookie authentication!
        },
        dataType: 'JSON'
    })
        .done(function(data, textStatus, jqXHR) {

            var processedData = processObjects(data, details);

            q.resolve(processedData);
        })
        .fail(function() {

            q.reject();
        });

    return q.promise;

    // Helper Functions
    // Construct the correct url for getting the data.
    function createUrl(uri, namespace, type) {

        // ex. https://api.redbookcloud/chops/resources/Store?paging=limit:20
        return uri + '/' + namespace + '/resources/' + type + '?paging=limit:20';
    }

    // Process objects so that Handlebars can consume it and create the html.
    function processObjects(objects, details) {

        var processed = [];

        // NOTE: this doesn't work for arrays or objects...
        _.each(objects, function(object) {

            console.log(object.sys_created_at);

            var processedObj = {
                title: object.display_name || object.name || object.sys_id,
                subTitle: moment(object.sys_created_at).format('MMM D YYYY h:mma'),
                details: []
            };

            _.each(details, function(value) {

                if (object[value]) {
                    processedObj.details.push({label: value, value: object[value]});
                }

            });

            processed.push(processedObj);
        });

        data = processed;   // Save

        return {items: processed};

    }

};

/**
 * Create the html for the lists based on the data retrieved from the server.
 * @param processedObjs
 */
var populateList = function(processedObjs) {

    // Using Handlebars to generate list view
    var source =
        '{{#items}}' +
            '<div class="listItem" onclick="clickListItem({{@index}});">' +
            '<div class="leftSide">' +
            '<div class="listTitle"> {{title}} </div>' +
            '<div class="listTime"> {{subTitle}} </div>' +
            '</div>' +
            '<div class="listGreaterThan"> > </div>' +
            '</div>' +
            '{{/items}}';

    var template = Handlebars.compile(source);

    $('.list').html(template(processedObjs));

};

/**
 * Create the html for the detail view based on the list item chosen.
 * @param processedObj
 */
var populateDetail = function(processedObj) {

    // Using Handlebars to generate detail view
    var source =
        '<div class="detailBig">{{title}}</div>' +
            '{{#details}}' +
            '<div class="detailNormal">' +
            '<div class="detailLabel">{{label}}</div>' +
            '<div class="detailValue">{{value}}</div>' +
            '</div>' +
            '{{/details}}';

    var template = Handlebars.compile(source);

    $('.detailContainer').html(template(processedObj));

};

// List View Click Events
var clickMenu = function() {
    window.mainMenu.showMenu();
};

/**
 * Switch to the detail view.
 * @param index
 */
function clickListItem(index) {

    $('#listView').hide();
    $('#detailView').show();
    populateDetail(data[index]);
}

// Detail View Click Events

/**
 * Switch to the list view.
 */
var clickBack = function() {

    $('#listView').show();
    $('#detailView').hide();
};

var showListErrorMessage = function(msg) {
    var source =
        '<div class="listItem">' +
            '<div class="leftSide">' +
            msg +
            '</div>' +
            '</div>';


    var template = Handlebars.compile(source);

    $('.list').html(template());
};



window.device.ready(function() {
    uri = window.device.apiURL || 'https://api.redbookcloud.com';

    /** MAIN **/
    getNamespace(uri)
        .then(function(namespace) {

            return getData(uri, namespace, type);
        }, function() { // getNamespace Fails
            showListErrorMessage('Error! Could not retrieve namespace.');
        })
        .then(function(data) {

            populateList(data);
        }, function() { // getData Fails
            showListErrorMessage('Error! Could not retrieve data.');
        });


});
