module.exports = function AppController(app){

    function validateApp() {

        var valid = false,
            msg = '';

        var validatedProfile = validateProfile(app.getProfile());

        if (!app.getName()) {

            msg = 'Missing name attribute in package.json.';
        } else if (!app.getVersion()) {

            msg = 'Missing version attribute in package.json.';
//        } else if (!validatedProfile.valid) {
//
//            msg = validatedProfile.msg;
        } else {

            valid = true;

            // provide defaults
            if (!app.getTitle()) {
                app.setTitle(app.getName());
            }
            if (!app.getDescription()) {
                app.setDescription(app.getName());
                console.log('Description has been set as: ', app.getDescription());
            }
        }

        return {valid: valid, msg: msg};

        /** HELPER FUNCTION **/
        function validateProfile(profile) {

            var valid = false;

            if (!profile) {

                msg = 'Missing profile in package.json.';
            } else if (
                typeof profile.dml !== 'object' ||
                profile.dml.constructor.toString().indexOf("Array") !== -1)   // TODO: check this logic
            {

                msg = 'profile.dml in package.json is required and must be an object.'
            } else if (!profile.name || typeof profile.name !== 'string') {

                msg = 'profile.name in package.json is required and must be a string.';
            } else {

                valid = true;
            }

            return {valid: valid, msg: msg};
        }

    }


    /*
    function createProfile(profileName) {

        // NOTE: cannot call createProfile more than once!
            // once created, its there forever...

        if (!app.getProfile()) {
            var profile = {
                name: profileName,
                dml: {
                    Application: {
                        select: {},
                        update: {},
                        insert: {},
                        delete: {},
                        aggregate: {}
                    }
                }
            };

            app.setProfile(profile);

            return {success: true, profile: profile};
        } else {

            return {success: false, message: 'Could not create profile. Has it already been created?'};
        }

    }
    */
    /*
    function changeProfileName(profileName) {

        var profile = app.getProfile();

        if (profile) {

            profile.name = profileName;

            app.setProfile(profile);

            return {success: true, profile: profile};
        } else {

            return {success: false, message: 'Could not change profile name. Has it already been created?'};
        }
    }
    */

    function viewProfile() {

        var profile = app.getProfile();

        if (!profile) {
            return {success: false, message: 'Profile not found. Has a profile been created?'};
        } else {
            return {success: true, profile: profile};
        }
    }

    function addTypeToProfile(type, actions) {

        var profile = app.getProfile();


        if (type === 'Application') {
            return {success: false, message: 'Cannot overwrite Application type.'};
        }

        if (profile &&
            typeof profile === 'object' &&
            profile.dml &&
            profile.dml.constructor.toString().indexOf("Array") === -1) {

            var typeDefinition = {};

            if (actions.select) {
                typeDefinition.select = {};
            }

            if (actions.delete) {
                typeDefinition.delete = {};
            }

            if (actions.update) {
                typeDefinition.update = {};
            }

            if (actions.insert) {
                typeDefinition.insert = {};
            }

            if (actions.aggregate) {
                typeDefinition.aggregate = {};
            }

            profile.dml[type] = typeDefinition;

            app.setProfile(profile);

            return {success: true, profile: profile};
        } else {

            return {success: false, message: 'Failed to add type to profile. Have you created a profile?'};
        }

    }

    // Cannot delete Application
    function removeTypeFromProfile(type) {

        var profile = app.getProfile();

        if (type === 'Application') {
            return {success: false, message: 'Cannot remove Application type.'};
        }


        if (profile && profile.dml) {

            delete profile.dml[type];
            return {success: true, profile: profile};
        } else {

            return {success: false, message: 'Failed to remove type from profile. Have you created a profile?'};
        }

    }

    return {
        validateApp           : validateApp,
//        createProfile         : createProfile,
//        changeProfileName     : changeProfileName,
        viewProfile           : viewProfile,
        addTypeToProfile      : addTypeToProfile,
        removeTypeFromProfile : removeTypeFromProfile
    };
};