module.exports = function RestController(project, options){

    var _       = require('../utils.js');
    var Client  = require('rbc-rest-client');
    var helper  = require('../helpers')(options)
      , Views   = require('../views/http');

    /**
     * Get the Default Output
     * @param verb
     * @returns {string}
     */
    function getDefaultOutput(verb){
        verb   = verb || '';
        var format = 'status';
        switch(verb.toUpperCase()){
            case 'PING':
                format = 'ping';
                break;
            case 'LOGIN':
                format = 'login';
                break;
            case 'GET':
                format = 'entity';
                break;
            case 'COUNT':
                format = 'entity';
                break;
            case 'PATCH':
                format = 'headers';
                break;
            case 'POST':
                format = 'location';
                break;
            case 'PUT':
                format = 'status';
                break;
            case 'DELETE':
                format = 'status';
                break;
            case 'HEAD':
                format = 'headers';
                break;
            case 'OPTIONS':
                format = 'headers';
                break;
            default:
                format = 'status';
                break;
        }

        return format;
    }

    /**
     * Get the View
     * @param output
     * @param verb
     * @returns {*}
     */
    function getView(verb, options){
        var view;
        var views  = Views(options);

        var output = (options.view) || getDefaultOutput(verb)
        if(typeof output === 'string'){
            output = output.toLowerCase()
        }

        //Create the Verb
        switch(output){
            case 'status':
                view = views.status;
                break;
            case 'headers':
                view = views.headers;
                break;
            case 'entity':
                view = views.entity;
                break;
            case 'full':
                view = views.full;
                break;
            case 'all':
                view = views.all;
                break;
            case 'ping':
                view = views.ping;
                break;
            case 'login':
            case 'auth':
                view = views.auth;
                break;
            case 'location':
                view = views.location;
                break;
            case 'id':
                view = views.id;
                break;
            case 'csv':
                view = views.csv;
                break;
            default:
                view = views.status;
        }
        return view;
    }

    /**
     * Request
     * @param env
     * @param verb
     * @param json
     * @param data
     * @param output
     */
    function request (env, verb, path, json, viewOptions){

        //convert the count
        if(verb === 'count'){
            verb = 'GET';
            path = [_.str.rtrim(path, '/'), 'count'].join('/');
        }

        env.createClient().request(verb, path, json, getView(verb, viewOptions))
    }


    return {
        request: request
    }
}