module.exports = function(options) {

    'use strict';

    options = options || {};

    function generate(type, pathToAppFolder, data) {

        var path = require('path'),
            _ = require('lodash'),
            Q = require('q');

        var q = Q.defer();

        var details = [];

        _.each(data, function(value, key) {
            if ((value.type === 'String' || value.type === 'Integer' || value.type == "DateTime") && !value.multi) {
                details.push(key);
            }
        });

        var pathToAppJsFile = path.join(pathToAppFolder, 'src/app.js');
        var pathToIndexHtmlFile = path.join(pathToAppFolder, 'src/index.html');

        Q.allSettled([

            template(pathToAppJsFile, pathToAppJsFile, {
                type: type,
                details: JSON.stringify(details)
            }),
            template(pathToIndexHtmlFile, pathToIndexHtmlFile, {
                type: type
            })
        ])
        .then(function() {

            q.resolve('App files created.');
        })
        .fail(function(err) {

            q.reject('Failed to create files.');
        });

        return q.promise;
    }

    function template(src, dest, replacementObj) {

        // Dependencies
        var Q = require('q');
        var fs = require('fs');
        var _ = require('lodash');

        var q = Q.defer();

        fs.readFile(src, function(err, packageObj) {

            if (err) {
                q.reject('Failed to read template. Please try again.\n' + err); // TODO: fix error msg
            }

            var compiledFile = _.template(packageObj, replacementObj);

            // write file
            fs.writeFile(dest, compiledFile, function(err) {
                if (err) {
                    q.reject('Failed to generate app. Please try again.');
                }
                q.resolve();    // TODO: resolve w/ msg
            });
        });

        return q.promise;
    }

    return {
        generate: generate
    };

};

//generate(userArgs['t'], userArgs['n'], userArgs['u'])



