module.exports = function(options){
    'use strict';

    options = options || {};


    // TODO: put these in the methods that require them!
    // TODO: promises?
    var output   = require('./views/console')(options);

    var fs       = require('fs-extra');
    var path     = require('path');
    var findup   = require('findup');
    var rimraf   = require('rimraf');
    var async    = require('async');
    var Q        = require('q');
    var http     = require('http');
    var promptAsync   = require('prompt');
//    var prompt   = require('sync-prompt').prompt;

    var config        = require('./config');
    var _             = require('./utils');
    var Project       = require('./project');
    var Environment   = require('./environment');
    var App           = require('./app');

    var PROJECT_FILE  = 'rbc-project.json';
    var PACKAGE_FILE  = 'package.json';

    var NullProject = {
        getDefaultEnvironment: function(){return null;}
        , hasEnvironment     : function(){ return false;}
    };

    function fatal(msg, code) {
        code = code || 5;
        console.log(msg.red);
        process.exit(code);
    }

    function loadProjectSync(){
        var project = new Project();
        try{
            var home = findup.sync(process.cwd(), PROJECT_FILE);
            try{
                if(options.verbose) console.log('Binding to project ' + home)
                process.chdir(home);
                var file       = path.join(home, PROJECT_FILE);
                var spec       = require(file);
                var project    = new Project(spec);
                if(options.verbose) console.log('Project Bound');
                if(options.debug)   console.log('Project', project);
            } catch(err){
                if(options.verbose) console.error('Unable to bind to ' + PROJECT_FILE +  ' in ' + home);
                if(options.debug)   console.error(err);
            }
        } catch(e) {
            if(options.verbose) console.error('Unable to open ' + PROJECT_FILE +  ' in ' + home);
            if(options.debug)   console.error(e);
        }
        return project;
    }

    function loadPackageSync(appHome) {

        try{
            process.chdir(appHome);
            var file       = path.join(appHome, PACKAGE_FILE);
            var attr       = fs.readJsonSync(file);
//            var attr       = require(file);
            var app        = new App(attr);
        } catch(err){
            this.fatal('Unable to bind to ' + PACKAGE_FILE +  ' in ' + appHome);
            if(options.debug)   console.error(err);
        }

        return app;
    }

    /**
     * Create
     * @param argOpts
     * @returns {{}}
     */
    function createEnvironmentPatch(args){

        args    = args    || {}

        var patch = {}

        //Create a diff
        if (args.uri)       patch.uri       = args.uri;
        if (args.username)  patch.username  = args.username;
        if (args.password)  patch.password  = args.password;
        if (args.namespace) patch.namespace = args.namespace;
        if (args.agent)     patch.agent     = args.agent;

        return patch;
    }

    function lookupEnvironment(project, args){

        project = project || NullProject;
        args    = args    || {}

        var name    = args.environment || project.getDefaultEnvironment();
        var env     = null;

        if(name == null || !project.hasEnvironment(name)){
            if(options.verbose) console.log('Unable to locate an environment');
            env = null;
        } else {
            if(options.verbose) console.log("Using environment " + name);
            env = project
                .getEnvironment(name)
                .patch(createEnvironmentPatch(args))
        }

        if(options.debug) console.log('Env', env);

        return env;
    }

    //add to the project file
    function createEnvironment(project, args){

        project = project || NullProject;
        args    = args    || {}

        var name = args.environment || project.getDefaultEnvironment();
        var env  = null;

        if(name == null || !project.hasEnvironment(name)){
            if(options.verbose) console.log('Using anonymous environment');
            env = new Environment('_$ANONYMOUS$_', {})
        } else {
            if(options.verbose) console.log("Using environment " + name);
            env = project.getEnvironment(name)
        }

        //Patching the environment
        env.patch(createEnvironmentPatch(args))

        if(options.debug) console.log('Env', env);

        return env;
    }


    // TODO: change to promises; fatal should be moved to app-tools
    // ASSUMPTION: publishDirectory is just the name of the dir from the root of the app folder
        // publishDirectory must be located in the root of the app folder!
        // if no publishDirectory => make the assumption it will be in www
        // ENCOURAGE: having production version of the code for performance
    function checkFile(appPath, publishDirectory, file) {

        file = file || '';

        var publishFolderPath = path.join(appPath, publishDirectory, file);

        try {

            return fs.openSync(publishFolderPath, 'r');
        } catch (e) {

            output.fatal(file + ' is not located at the root of ' + publishDirectory + '/');

            return false;
        }
    }

    // require in the fn being tested doesn't play nice w/ mock-fs!
        // therefore moved out of function
    var archiver = require('archiver');
    function createAppZip(name, appPath, publishDirectory, zipIdentifier) {

        // Dependencies
        var archive  = archiver('zip');

        // Fn vars
        var q = Q.defer();

        // Write zip to file
        var zipName = name + '-' + zipIdentifier + '.zip',
            output = fs.createWriteStream(path.join(appPath, zipName));
        output.on('close', function() {
            q.resolve({size: archive.pointer(), zipName: zipName, msg: zipName + ' has been written to ' + appPath + '/'});
        });
        output.on('error', function(err) {
            q.reject('Failed to create zip file'); // not sure if need
        });

        // Create zip
        archive.pipe(output);
        archive.bulk([
            {expand: true, cwd: path.join(appPath, publishDirectory), src: ['**']}
        ]);
        archive.finalize();

        return q.promise;
    }

    // TODO: refactor - make this cleaner, more organized!
            var request = require('request');
    function sendApp(zipName, size, appPath, credentials, app) {

        // Dependencies
        var node_url = require('url');

        var q = Q.defer(),
            username = credentials.username,
            password = credentials.password,
            uri = credentials.uri,
            namespace = credentials.namespace,
            zipPath = path.join(appPath, zipName),
            name = getAppName(zipName),
            urlPath = path.join(namespace, 'apps'),
            self = this;

        if (size > config.MAX_ZIP_BYTES) {

            q.reject({error: 'zip file size too big (Max zip file size: 20mb).'});
        } else {

            // Get form ready
            var formData = {
                upload: {
                    value: fs.createReadStream(zipPath),
                    options: {
                        filename: name + '.zip',
                        contentType: 'application/zip',
                        knownLength: size
                    }
                },
                name: app.getName(),
                title: app.getTitle(),
                version: app.getVersion(),
                description: app.getDescription()
            };

            // TODO: implement progress bar for this
            // like int fetchArchive

            checkIfPOSTorPUT(urlPath, name, uri, username, password)
                .then(function(response) {

                    var type = response.type,
                        postUrl = response.postUrl;

                    output.info('Submitting...');

                    output.debug(postUrl);
                    output.debug(type);
                    output.debug(formData);
                    output.debug(username);
                    output.debug(password);

                    return publish(postUrl, type, formData, username, password);

                }, function(response) {

                    cleanUpPublish(zipPath);    // TODO: this is async => plan for this with promises...
                    q.reject(response);
                })
                .then(function(response) {  // finish publish
                    output.debug(response);
                    output.debug(response.msg);
                    output.debug(response.type);

                    q.resolve(response);
                }, function(response) {

                    q.reject(response);
                });
        }

        return q.promise;

        /** HELPER FUNCTIONS **/
        function getAppName(zipName) {
            var zipNameSplit = zipName.split('-');

            if (zipNameSplit.length > 1) {
                zipNameSplit.pop();     // Take out the last segment (date that was tacked on in createAppZip)
                return zipNameSplit.join('-');
            } else {
                return zipNameSplit[0];
            }
        }

        function checkIfPOSTorPUT(urlPath, name, uri, username, password) {

            var q = Q.defer();

            var url = node_url.resolve(uri, path.join(urlPath, name));
            request({
                method: 'GET',
                uri: url,
                auth: {
                    user: username,
                    password: password
                }
            }, function(err, response, body) {

                if (err) {

                    output.verbose(err);
                    q.reject({error: 'Failed to publish app. Aborting publish.'});
                } else {

                    var type,
                        postUrl;

                    output.debug('CHECK GET: ' + response.statusCode);
                    output.debug(url);

                    if (response.statusCode !== 200) {

                        type = 'POST';
                        postUrl = 'https://' + node_url.parse(uri).host + '/' + urlPath;
                        q.resolve({type: type, postUrl: postUrl});
                    } else {

                        type = 'PUT';
                        postUrl = 'https://' + node_url.parse(uri).host + '/' + path.join(urlPath, name);

                        output.debug('body of check: ' + body);


                        if (JSON.parse(body).developer !== username) {

                            /*
                            var overwrite = prompt('This app has already been published by ' + JSON.parse(body).developer.toString() + ".\nDo you want to overwrite it [yes or no]?    \n".cyan);
                            if (overwrite === 'yes' || overwrite === 'y') {

                                q.resolve({type: type, postUrl: postUrl});
                            } else {

                                q.reject({error: 'Aborting publish.'});
                            }
                            */

                            promptAsync.message = 'app-tools';
                            promptAsync.start();
                            promptAsync.get([
                                {
                                    name: "overwrite",
                                    description: 'This app has already been published by ' + JSON.parse(body).developer.toString() + ".\nDo you want to overwrite it [yes or no]?    \n".cyan,
//                                    description: "This app has already been published by ".cyan,
                                    required: true
                                }
                            ], function(err, result) {

                                if (err) {

                                    output.verbose('Error occurred: ' + err);

                                    q.reject({error: 'Aborting publish.'});
                                } else {

                                    var overwrite = result.overwrite;

                                    if (overwrite === 'yes' || overwrite === 'y') {

                                        q.resolve({type: type, postUrl: postUrl});
                                    } else {

                                        q.reject({error: 'Aborting publish.'});
                                    }
                                }
                            });

                        } else {

                            q.resolve({type: type, postUrl: postUrl});
                        }
                    }
                }
            });

            return q.promise;
        }

        function publish(postUrl, type, formData, username, password) {

            var q = Q.defer();

            request({
                uri: postUrl,
                auth: {
                    user: username,
                    password: password
                },
                method: type,
                formData: formData
            }, function(err, response, body) {

                cleanUpPublish(zipPath);    // TODO: this is async => plan for this with promises...

                output.debug('postUrl: ' + postUrl);
                output.debug('method: ' + type);
                output.debug('code: ' + response.statusCode || 'none');
                output.debug('body: ' + body);
                output.debug(response);

                if (err || type === 'PUT' && response.statusCode !== 204 || type === 'POST' && response.statusCode !== 201) {

                    if (err) output.info('Error occurred: ', err);

                    // TODO: NOTE THIS IS REPEAT => FIX
                    try {
                        if (body) {
                            var errorMessage = JSON.parse(body);    // this sometimes causes problems...
                            q.reject({error: errorMessage + '\nFailed to publish app to ' + postUrl + '. Aborting publish.'});
                        } else {
                            q.reject('Failed to publish app to ' + postUrl + '. Aborting publish.');
                        }

                    } catch(e) {

                        q.reject({error: 'Failed to publish app to ' + postUrl + '. Aborting publish.'});
                    }

                } else {

                    q.resolve({type: type, message: name + ' published to ' + postUrl});
                }

            });

            return q.promise;
        }
    }

    function assignProfile(profileName, usernameToAssign, credentials) {
        var node_url = require('url');

        var q = Q.defer(),
            namespace = credentials.namespace,
            username = credentials.username,
            password = credentials.password,
//            url = credentials.uri + '/' + path.join(namespace, 'users', usernameToAssign, 'profiles', 'add'),
            url = node_url.resolve(credentials.uri, path.join(namespace, 'users', usernameToAssign, 'profiles', 'add'));

        request({
            method: 'PATCH',
            uri: url,
            auth: {
                user: username,
                password: password
            },
            body: {
                profiles: [profileName]
            },
            json: true
        }, function(err, response, body) {


            output.debug(response.statusCode);
            output.debug(body);

            if (err || response.statusCode !== 204) {

                output.verbose(err);

                // TODO: this
                q.resolve('Could not assign profile to ' + usernameToAssign + '.' +
                    ' Please run the following command with a ' +
                    'set of credentials that allows you to add profiles to users: \n\n' +
                    'app-tools assign-profile-to-user <username>');
            } else {

                q.resolve(usernameToAssign + ' now has ' +  profileName + ' as a profile. \n' +
                    'Login as ' + usernameToAssign + ' to see this app.');
            }

        });

        return q.promise;
    }

    function createCustomApp(name, title, description, pathToAppFolder) {

        var q = Q.defer();

        fs.copy(path.join(__dirname, 'custom-templates'), pathToAppFolder, function(err) {

            if (err) {

                output.verbose('Error occurred: ' + err);
                output.verbose('Cleaning up files...');

                cleanUpNewApp(pathToAppFolder)
                    .then(function() {

                        q.reject('Failed to write files to ' + pathToAppFolder + ' \nAborted app creation.');
                    });
            }

            // TODO: prompt goes here!

            writeToPackageJson(name, title, description, pathToAppFolder, path.join(pathToAppFolder, 'package.json'))
                .then(function(msg) {

                    output.verbose(msg);
                    q.resolve('Files have been written to ' + pathToAppFolder);
                })
                .fail(function(msg) {

                    // Question: what about profile - user shouldn't edit this (need to hold to the convention that
                        // name and profile name should be the same...
                        // => maybe REJECT!?!?!?
                    output.verbose(msg);
                    q.resolve('Failed to update package.json with name, title, and description. Please update it manually.');
                });

        });

        return q.promise;

    }

    function createAppGeneratorApp(type, name, title, description, pathToAppFolder, data) {

        var q = Q.defer();

        var generate = require('./generate-app.js')(options).generate;

        // Copy template files to user's app directory
        fs.copy(path.join(__dirname, 'app-generator-templates'), pathToAppFolder, function(err) {

            if (err) {

                output.verbose('Error occurred: ' + err);
                output.verbose('Cleaning up files...');

                cleanUpNewApp(pathToAppFolder)
                    .then(function() {

                        q.reject('Failed to write files to ' + pathToAppFolder + ' Aborted app creation.');
                    });
            }

            output.verbose('Files have been written to ' + pathToAppFolder);

            writeToPackageJson(name, title, description, pathToAppFolder, path.join(pathToAppFolder, 'package.json'))
                .then(function(msg) {

                    // Notify write successful
                    output.verbose(msg);

                    return generate(type, pathToAppFolder, data);
                })
                .then(function(msg) {

                    // Notify generate successful
                    output.verbose(msg);

                    q.resolve('Successfully wrote app files to ' + pathToAppFolder + '. \nApp built for type: ' + type + '.');
//                    q.resolve('App built for type: ' + type + '.');
                })
                .fail(function(errorMessage) {

                    cleanUpNewApp(pathToAppFolder)
                        .then(function(msg) {

                            // Notify cleaned up
                            output.verbose(msg);

                            // Notify could not write file
                            q.reject(errorMessage);
                        }, function(msg) {

                            // Notify could not clean up app
                            output.error(msg);

                            // Notify could not write file
                            q.reject(errorMessage);
                        });
                });
        });

        return q.promise;

    }

    // TODO: should return a promise!
        // helpers should start and end in app-tools.js not in helpers.js...
    // TODO: helpers should have fatal, info, and debug! (user view.js?!?!?!)
    function createAngularSeedApp(name, title, description, pathToAppFolder) {

        var q = Q.defer();

        fetchArchive(pathToAppFolder, config.ANGULAR_ENDPOINT)
            .then(function() {

                output.verbose('Angular files fetched.');
                return writeToPackageJson(name, title, description, pathToAppFolder, path.join(pathToAppFolder, 'package.json'));
            })
            .then(function(msg) {

                output.verbose(msg);
                q.resolve('Angular seed project created.');
            })
            .fail(function(msg) {

                if (msg) output.verbose(msg);

                var errMsg = 'Failed to create angular seed app.';

                cleanUpNewApp(pathToAppFolder)
                    .then(function(msg) {

                        output.verbose(msg);
                        output.fatal(errMsg);
                    }, function(msg) {

                        output.verbose(msg);
                        output.fatal(errMsg);
                    });
            });

        return q.promise;
    }

    // HELPER

    // NOTE: used ionic-cli's fetchArchive function
    function fetchArchive(targetPath, archiveUrl) {

        var os = require('os');
        var q = Q.defer();
        var unzip = require('unzip2');

        console.log('\nDownloading:', archiveUrl);

        var tmpFolder = os.tmpdir();
        var tempZipFilePath = path.join(tmpFolder, 'angular-seed-' + new Date().getTime() + '.zip');

        // HELPER
        var unzipRepo = function(fileName) {

            var readStream = fs.createReadStream(fileName);
            readStream.on('error', function(err) {
                console.log( ('unzipRepo readStream: ' + err).red );
                q.reject(err);
            });

            readStream.on('open', function() {
                readStream.pipe(unzip.Parse())
                    .on('entry', function(entry) {
                        var fileName = entry.path.split('/').slice(1).join('/');

                        if (fileName === "") {
                            entry.autodrain();
                        } else if (entry.type === 'Directory') {
                            fs.ensureDir(path.join(targetPath, fileName));
                        } else {    // file
                            entry.pipe(fs.createWriteStream(path.join(targetPath, fileName)));
                        }
                    })
                    .on('close', function() {
                        console.log('Successfully wrote app files to ' + targetPath);
                        q.resolve();
                    })
                    .on('error', function() {
                        // TODO: add clean up new app to all fatals!
                        q.reject('Failed to write app files to ' + targetPath);
                    });
            });

        };

        // MAIN
        var proxy = process.env.PROXY || process.env.http_proxy || null;
        var request = require('request');
        request({ url: archiveUrl, rejectUnauthorized: false, encoding: null, proxy: proxy }, function(err, res, body) {
            if(err) {
                console.error('Error fetching:'.red, archiveUrl, err);
                q.reject(err);
                return;
            }
            if(!res) {
                console.error('Invalid response:'.red, archiveUrl);
                q.reject('Unable to fetch response: ' + archiveUrl);
                return;
            }
            if(res.statusCode !== 200) {
                if(res.statusCode === 404 || res.statusCode === 406) {
                    console.error('Not found:'.red, archiveUrl, '(' + res.statusCode + ')');
                    console.error('Please verify the url and try again.'.red);
                } else {
                    console.error('Invalid response status:'.red, archiveUrl, '(' + res.statusCode + ')');
                }
                q.reject(res);
                return;
            }
            try {
                fs.writeFileSync(tempZipFilePath, body);
                unzipRepo(tempZipFilePath);
            } catch(e) {
                console.log('fetchArchive request write: ' + e);
                q.reject(e);
            }
        }).on('response', function(res){
                var ProgressBar = require('progress');
                var bar = new ProgressBar('[:bar]  :percent  :etas', {
                    complete: '=',
                    incomplete: ' ',
                    width: 30,
                    total: parseInt(res.headers['content-length'], 10)
                });

                res.on('data', function (chunk) {
                    try {
                        bar.tick(chunk.length);
                    } catch(e){}
                });
            });

        return q.promise;
    }

    function writeToPackageJson(name, title, description, pathToAppFolder, srcPackageJson) {

        var q = Q.defer();

        fs.readFile(srcPackageJson, function(err, packageObj) {

            if (err) {

                output.verbose('Error occurred: ' + err);

                q.reject('Failed to read package.json template. Please try again.\n');
            }

            var compiledJson = _.template(packageObj, {
                name: name,
                title: title,
                description: description
            });

            // write file
            fs.writeFile(path.join(pathToAppFolder, '/package.json'), compiledJson, function(err) {

                if (err) {

                    output.verbose('Error occurred: ' + err);

                    q.reject('Failed to write package.json.');
                }

                q.resolve('package.json has been written to ' + pathToAppFolder);
            });
        });

        return q.promise;
    }

    // TODO: test
    function cleanUpNewApp(pathToAppFolder) {

        var q = Q.defer();

        // Delete app directory
        rimraf(pathToAppFolder, function(err) {

            if (err) {

                output.verbose('Error occurred: ' + err);
                q.reject('Could not delete ' + pathToAppFolder + '. Please manually delete it.')
            }

            q.resolve('Cleaned up files from ' + pathToAppFolder + ' .');
        });

        return q.promise;

    }

    // TODO: test
    function cleanUpPublish(zipPath) {

        fs.unlink(zipPath, function(err) {
            if (err) {
                output.verbose('Error occurred: ' + err);
                output.fatal('Could not delete app zip. Please manually delete it.');
            }

            output.verbose('Finished cleanup.');
        });
    }

    function getPathToAppFolder(cwd) {

        // Find name of app
        var split = cwd.split('/'),
            index = split.indexOf('apps'),
            lastIndex = split.length-1;

        if (index > 0 &&                // apps exist in path
            lastIndex - index > 0) {    // not in apps folder
            var name = split[index+1];  // right after apps
        } else {
            fatal('Not in an app directory! Please cd into your app directory and try again.');
        }

        return path.join(process.cwd(), '/apps/', name);

    }

    // QUESTION: should this check remotely?!?
    function checkIfAppExists(pathToAppFolder) {

        return fs.existsSync(pathToAppFolder);
    }

    function createAppDirectory(pathToAppFolder) {
        // Create project directory
        fs.mkdirpSync(pathToAppFolder);   // TODO: make this async
    }

    function profileHelperCurrier(type) {

        // add namespace if profileName doesn't specify one?

        return function(name, credentials, profileName) {

            // dependencies
            var node_url = require('url');
            var prettyJSON = require('prettyjson');

            var request = require('request'),
                q = Q.defer(),
                namespace = credentials.namespace,
                username = credentials.username,
                password = credentials.password,
                url;

            if (type === 'get') {
                url = node_url.resolve(credentials.uri, path.join(namespace, 'apps', name));
//                url = credentials.uri + path.join(namespace, 'apps', name);
//                url = 'https://' + path.join(config.SERVER_ENDPOINT, 'apps', name);
            } else {
                url = node_url.resolve(credentials.uri, path.join('apps', name, 'profiles', profileName));
//                url = credentials.uri + path.join(namespace, 'apps', name, 'profiles', profileName);
//                url = 'https://' + path.join(config.SERVER_ENDPOINT, 'apps', name, 'profiles', profileName);
            }

            request({
                method: type,
                uri: url,
                auth: {
                    user: username,
                    password: password
                }
            }, function(err, response, body) {

                output.debug('namespace:', namespace);
                output.debug('url:', url);
                output.debug('username:', username);
                output.debug('body:', body);
                output.debug('status code:', response.statusCode);

                if (err) {

                    output.verbose(err);
                    q.reject('Failed to retrieve profile.');
                } else if (type === 'get' && response.statusCode === 404) {

                    q.reject('No record of this app exists in the cloud. Did you publish this app yet?');
                } else {

                    if (type === 'get') {

                        q.resolve(JSON.parse(body).profile);
//                        q.resolve(prettyJSON.render(JSON.parse(body).profiles));
                    } else {
                        q.resolve();
                    }
                }

                /*
                if (err || ( ((type === 'post' || type === 'delete') && response.statusCode !== 204)) || (type === 'get' && response.statusCode !== 200)) {

                    if (type === 'get' && response.statusCode === 404) {
                        q.reject('No record of this app exists in the cloud. Did you publish this app yet?');
                    }

                    q.reject(err);
                } else {

                    // REMOVE
//                    console.log(JSON.parse(JSON.parse(body)));
                    // REMOVE

                    if (type === 'get') { // TODO: make pretty  q.resolve(JSON.parse(body).profile);
//                        q.resolve(prettyJSON.render(JSON.parse(body).profiles));
                    } else {
                        q.resolve();
                    }

                }
                */

            });

            return q.promise;
        };
    }

    function validateCredentials(project, argv) {
        var env = createEnvironment(project, argv);

        var namespace = env.getNamespace(),
            username = env.getUser(),
            password = env.getPassword(),
            uri = env.getUri();

        if (!namespace || !username || !password || !uri) {
            output.fatal('Please provide an environment or a namespace, username, password, and uri.');
        }

        return {
            namespace: namespace,
            username: username,
            password: password,
            uri: uri
        };
    }

    function listApps(credentials) {

        // Dependencies
//        var prettyJSON = require('prettyjson');
        var node_url = require('url');

        var request = require('request');
        var namespace = credentials.namespace,
            username = credentials.username,
            password = credentials.password,
            q = Q.defer();

        var url = node_url.resolve(credentials.uri, path.join(namespace, 'apps'));
//        var url = 'https://' + path.join(config.SERVER_ENDPOINT, 'apps');

        request({
            method: 'GET',
            uri: url,
            auth: {
                user: username,
                password: password
            }
        }, function(err, response, body) {

            if (err || response.statusCode !== 200) {
                q.reject();
            } else {
                q.resolve(JSON.parse(body, null, 4));
            }

            /*
            function process(body) {
                var processedBody = '\n';

                _.each(body, function(app) {

//                    app = JSON.parse(app);

//                    delete app.rbc_modified_at;
                    delete app.sys_type_version;
                    delete app.sys_type_version;
                    delete app.sys_modified_by;
                    delete app.sys_created_by;
//                    delete app.sys_created_at;
                    delete app.sys_created_namespace;

                    processedBody = processedBody + prettyJSON.render(app) + '\n\n';

                });

                return processedBody;
            }
            */

        });

        return q.promise;

    }

    // TODO: abstract this further?
        // abstract the credentials and return of promise
    function getAppMetadata(credentials, name) {

        var request = require('request'),
            node_url = require('url');

        var namespace = credentials.namespace,
            username = credentials.username,
            password = credentials.password,
            q = Q.defer();

        var url = node_url.resolve(credentials.uri, path.join(namespace, 'apps', name));

        request({
            method: 'GET',
            uri: url,
            auth: {
                user: username,
                password: password
            }
        }, function(err, response, body) {

            if (err) {

                output.verbose('Error occurred: ' + err);
                q.reject('Could not retrieve app metadata from server.');
            } else if (response.statusCode !== 200) {

                output.debug(url);
                output.debug(response.statusCode);
                output.debug(body);
                q.reject('Could not retrieve app metadata from server.');
            } else {

                q.resolve(JSON.parse(body, null, 4));
            }
        });

        return q.promise;
    }

    function removeAppRemote(credentials, name) {
        // Dependencies
        var request = require('request');

        var namespace = credentials.namespace,
            username = credentials.username,
            password = credentials.password,
            q = Q.defer();

//        var url = credentials.uri + path.join('apps', name);
        var url = node_url.resolve(credentials.uri, path.join('apps', name));
//        var url = 'https://' + path.join(config.SERVER_ENDPOINT, 'apps', name);

        request({
            method: 'DELETE',
            uri: url,
            auth: {
                user: username,
                password: password
            }
        }, function(err, response, body) {
            console.log(err);
            console.log(response.statusCode);
            if (err || response.statusCode !== 204) {
                q.reject();
            } else {
                q.resolve();
            }
        });

        return q.promise;
    }

    function getType(type, credentials) {

        var node_url = require('url');

        var q = Q.defer(),
            uri = credentials.uri,
            namespace = credentials.namespace,
            username = credentials.username,
            password = credentials.password;

        request({
            method: 'GET',
            uri: node_url.resolve(uri, path.join(namespace, 'types', type)),
            auth: {
                user: username,
                password: password
            }
        }, function(err, response, body) {

            if (err) {

                output.verbose(err);
                q.reject('Failed to find type: ' + type + '. Aborted app creation. \nAre you sure the specified type is defined in the cloud?');
            } else if (response.statusCode !== 200) {

                q.reject('Failed to find type: ' + type + '. Aborted app creation. \nAre you sure the specified type is defined in the cloud?');
            } else {

                // get type schema
                var data = JSON.parse(body).properties;
                q.resolve(data);
            }
        });

        return q.promise;
    }

    function promptForNewApp() {

        var q = Q.defer();

        promptAsync.message = 'app-tools';
        promptAsync.start();
        promptAsync.get([
            {
                name: "title",
                description: "title (display name) of app (required)".cyan,
                required: true
            },
            {
                name: "description",
                description: "description of app (optional)".cyan,
                required: false
            }
        ], function(err, result) {

            if (err) {

                output.verbose('Error occurred: ' + err);

                q.reject('\nAborted app creation.');
            } else {
                q.resolve(result);
            }
        });

        return q.promise;
    }

    return {
          fatal           : fatal
        , loadProjectSync : loadProjectSync
        , createEnv       : createEnvironment
        , lookupEnv       : lookupEnvironment
        , createEnvPatch  : createEnvironmentPatch

        // package.json
        , loadPackageSync : loadPackageSync

        // New App
        , createAppDirectory: createAppDirectory

        // Skeleton Apps
        , createCustomApp : createCustomApp
        , createAngularSeedApp : createAngularSeedApp
        , createAppGeneratorApp: createAppGeneratorApp

        // Checks
        , checkFile : checkFile

        // App Zip
        , createAppZip    : createAppZip

        // Associate App
        , addProfile      : profileHelperCurrier('post')
        , removeProfile   : profileHelperCurrier('delete')
        , listProfile    : profileHelperCurrier('get')

        // Publish App
        , getPathToAppFolder: getPathToAppFolder
        , sendApp         : sendApp
        , assignProfile: assignProfile

        // Remove App
        , removeAppRemote: removeAppRemote

        // Remote Info
        , listApps         : listApps
        , getAppMetadata   : getAppMetadata

        // Helpers
        , validateCredentials: validateCredentials
        , checkIfAppExists: checkIfAppExists
        , getType         : getType
        , promptForNewApp : promptForNewApp

        // Files
        , PROJECT_FILE    : PROJECT_FILE
        , PACKAGE_FILE    : PACKAGE_FILE
    }
};