/**
 *
 * Set of standard rest callbacks
 *
 * @type {{noop: Function, status: Function, headers: Function, entity: Function, full: Function, selectCallback: Function}}
 */

;module.exports = function(spec){
    'use strict';

    spec = spec || {};

    var util    = require('util');
    var http    = require('http');
    var _       = require('../utils');

    var inspect = require('eyes').inspector({
        styles: {                 // Styles applied to stdout
            all:     'cyan',      // Overall style applied to everything
            label:   'underline', // Inspection labels, like 'array' in `array: [1, 2, 3]`
            other:   'inverted',  // Objects which don't have a literal representation, such as functions
            key:     'bold',      // The keys in object literals, like 'a' in `{a: 1}`
            special: 'red',      // null, undefined...
            string:  'yellow',
            number:  'green',
            bool:    'green',      // true false
            regexp:  'green'      // /\d+/
        },
        pretty       : !spec.flat,           // Indent object literals
        hideFunctions: false,                // Don't output functions at all
        stream       : process.stdout,       // Stream to write to, or null
        maxLength    : spec.buffer || 4048   // Truncate output if longer
    });

    function render(content, overloads){

        overloads = overloads ||{};
        if(spec.color){
            inspect(content);
        } else {
            console.log(content);
        }
    }

    return {
        render      : render
    }
}